/******************************************************************************/
/* cd_comnd.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"
#include "cd_bind.h"
#include "cd_dsp.h"
#include "cd_host.h"
#include "st_hard.h"

/* -------------------------------------------------------------------------- */

#define	SEEKBUFF	1

#define	ONEREADS	1
#define	TWOREADS	0

/* -------------------------------------------------------------------------- */

#define	HZ_200	0x000004BAl

/* -------------------------------------------------------------------------- */

extern	CDROM	*gl_actdocu;

/* -------------------------------------------------------------------------- */
/* default device id */

extern	unsigned int	cd_defdevno;

/* different CDROM latency */

extern	unsigned int	cd_spinup;
extern	unsigned int	cd_spinup_addsession;
extern	unsigned int	cd_spinup_addtrack;

extern	unsigned int	cd_stopdisc;
extern	unsigned int	cd_stopdisc_middle;
extern	unsigned int	cd_stopdisc_end;

extern	unsigned int	cd_pause_toready;
extern	unsigned int	cd_unpause_toready;

extern	unsigned int	cd_shortseek_1sthalf;
extern	unsigned int	cd_shortseek_2ndhalf;

extern	unsigned int	cd_longseek_00to20;
extern	unsigned int	cd_longseek_21to40;
extern	unsigned int	cd_longseek_41to60;
extern	unsigned int	cd_longseek_61to74;

extern	unsigned int	cd_longseek_00to74;
extern	unsigned int	cd_longseek_00to74_addsession;

extern	unsigned int	cd_longseek_00to35;
extern	unsigned int	cd_longseek_00to35_addsession;

/* different lead in/out and pause time periods for CDROMs */

extern	unsigned long	cd_cdrmlead;
extern	unsigned long	cd_cdrmtail;
extern	unsigned long	cd_sesslead;
extern	unsigned long	cd_sesstail;
extern	unsigned long	cd_traklead;
extern	unsigned long	cd_traktail;

/* preload buffer size during seek and data rate from SCSI drive */

/*==============================================*/
#if SEEKBUFF > 0
/*==============================================*/

extern	unsigned long	cd_preload_buffsize;
extern	unsigned long	cd_preload_readrate;

/*==============================================*/
#endif
/*==============================================*/

/* -------------------------------------------------------------------------- */

static	int	disc_spinning;		/* whether the disk is turning */

static	int	pause_indicator;	/* PAUSE mode flag */
static	int	pause_mode;		/* PAUSE mode indicator */

#define	PLAY_NONE	0x0000		/* no data play back */
#define	PLAY_FILE	0x0001		/* play a data file */
#define	PLAY_ZERO	0x0002		/* play zeros */
#define	PLAY_BUFF	0x0004		/* play from seek buffer */

static	int	play_flag;		/* if TRUE play otherwise nothing */
static	ITEM	*curr_item;		/* current item being played */

static	int	curr_error;		/* last CD-Module error value */

static	unsigned long	curr_absqpos;	/* current absolute position in bytes */
static	unsigned long	curr_lastpos;	/* last position to be played */

static	unsigned long	item_endpos;	/* end position of the current data file */
static	unsigned long	item_padding;	/* padding after a file item */

static	unsigned int	buff_length;	/* length of the disk buffer */

/* -------------------------------------------------------------------------- */

static	unsigned int	goto_mins, goto_secs, goto_frms;
static	unsigned int	abplay_amins, abplay_asecs, abplay_afrms;
static	unsigned int	abplay_bmins, abplay_bsecs, abplay_bfrms;

/* play routine will start playing from given position to given position */

static	unsigned long	play_frframes;	/* play from this position */
static	unsigned long	play_toframes;	/* play to this position */

/* the seek time */

static	unsigned long	play_seektime;	/* seek time before play */

/* -------------------------------------------------------------------------- */

static	void	*savessp;

/******************************************************************************/
/******************************************************************************/

#include "cd_comnd.h"

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_comnd_emulate()
{
/* switch to superviser mode and initialise different */
/* global parameters used for emulation */

	savessp = Super(NULL);
	if ((gl_actdocu->NoItems > 0) && cd_comnd_initialise())
	{

/* start the emulation phase with receiving commands from */
/* JAGUAR development systom and responding to */
/* quit emulation if ESCape key pressed */

		for (;;)
		{
			if (cd_comnd_checkkbd())	break;
			if ((unsigned int)CH_DSA() == 0)
			{
				RX_DSA();
				if (DSA_err != 0)	break;
				(*cd_comnd_functable[((DSA_in >> 8) & 0x00FF)])(DSA_in & 0x00FF);
			}
			if ((play_flag != PLAY_NONE) && !pause_mode)
				cd_comnd_readdata();
		}
	}

/* switch back to user mode */

	Super(savessp);
	return;
}

/* -----------------------------------------------------------------------------
 */

static	int	cd_comnd_checkkbd()
{
register long	ch;
register int	ascii;

	if (!Bconstat(BC_CON))
		return(FALSE);

	ch = Bconin(BC_CON);
	ascii = (int)(ch & 0x000000FF);
	return((ascii == 27) ? TRUE : FALSE);
}

/* -----------------------------------------------------------------------------
 */

static	int	cd_comnd_initialise()
{
register CDROM	*docu = gl_actdocu;
register ITEM	*item;
register long	err;
FSELDTA	theDTA;
unsigned char	pathname[ MAX_PATHNAME ];

/* get the length and starting cluster of each file */
/* and update Q-time subcodes for different items */

	if (!fsel_initio(cd_defdevno))
		return(FALSE);

	for (item = docu->FirstItem ; item ; item = item->next)
	{
		if ((item->Settings & IT_TYPE) != IT_ITEM)
			continue;

		strcpy(pathname, item->PathName);
		err = fsel_findfile(pathname, &theDTA);
		if (err <= 0)
		{
			item->Cluster = 0;
			item->Length = 0L;
		}
		else
		{
			item->Cluster = theDTA.d_fat;
			item->Length = theDTA.d_fsize;
		}
	}
	cd_update_time(docu);

/* initialise different parameters used here */

	disc_spinning = FALSE;		/* initially the disk is not turning */

	pause_indicator = FALSE;	/* initially pause mode indicator if off */
	pause_mode = FALSE;		/* initially the CD is not in pause mode */

	play_flag = PLAY_NONE;		/* play back if off */
	curr_item = NULL;		/* current item being played */

	curr_error = COMND_E_NOERROR;	/* current CD-Module error value */

	curr_absqpos = 0l;		/* current absolute position in bytes */

/* load dsp program for I2S data bus */

	return(dsp_loadprg());
}

/******************************************************************************/
/******************************************************************************/

static	void	cd_comnd_playtitle(unsigned int param)		/* 01h */
{
register ITEM	*item;
register int	trakno;

	cd_comnd_playreset();			/* reset the current play stuff */
	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	trakno = 0;				/* find the specified track */
	for (item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if (!(item->Settings & IT_TRACK)) continue;
		if ((++trakno) == param) break;
	}
	if (item == NULL)
	{					/* return error if track does not exist */
		cd_comnd_senderror(COMND_E_ILLVALUE);
		return;
	}
	play_frframes = item->FrameStart;	/* set the play start position */
	cd_comnd_seek();			/* seek to the specified track */
	cd_comnd_setlast(0);			/* set the play end position */
	cd_comnd_playset();			/* set the current play stuff */
	cd_comnd_respond(COMND_FOUND);
	return;
}

static	void	cd_comnd_stop(unsigned int param)		/* 02h */
{
	cd_comnd_playreset();			/* reset the current play stuff */
	play_frframes = 0;			/* set head position to begining of disc */
	cd_comnd_seek();			/* seek to the specified position */
	cd_comnd_setlast(0);			/* set the play end position */
	cd_comnd_discspin(FALSE);		/* stop the disc if spinning */
	pause_indicator = FALSE;		/* clear the pause mode indicator */
	cd_comnd_respond(COMND_STOPPED);	/* send the STOPPED command */
	return;
}

static	void	cd_comnd_readtoc(unsigned int param)		/* 03h */
{
register ITEM	*track, *item;
register int	sessno;
register int	min_trakno, max_trakno;
register unsigned long	start;

	cd_comnd_playreset();			/* reset the current play stuff */
	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	sessno = -1, min_trakno = 0;		/* look for the specified session */
	for (item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if ((item->Settings & IT_SESSION) && ((++sessno) == param)) break;
		if (item->Settings & IT_TRACK) min_trakno ++;
	}
	if (item == NULL)
	{					/* specified session number does not exist */
		cd_comnd_senderror(COMND_E_ILLVALUE);
		return;
	}

	/* find the minimum/maximum track numbers in this session */
	max_trakno = min_trakno;
	min_trakno ++;				/* minimum track number in session */
	track = item->next;			/* pointer to first track in session */
	for (item = track ; item ; item = item->next)
	{
		if (item->Settings & (IT_SESSION | IT_LAST)) break;
		if (item->Settings & IT_TRACK) max_trakno ++;
	}
	play_frframes = track->FrameStart;	/* set the play start position */
	cd_comnd_seek();			/* seek to the specified track */
	cd_comnd_setlast(0);			/* set the play end position */
	cd_comnd_playset();			/* set the current play stuff */
	pause_mode = TRUE;			/* the CD Module goes into pause mode */

	/* respond with following info about the specified session */
	/*	first title number, last title number, start */
	/*	time (minutes:seconds:frames) of first track */
	cd_comnd_respond(COMND_TOCVAL_MINTRACK | min_trakno);
	cd_comnd_respond(COMND_TOCVAL_MAXTRACK | max_trakno);
	start = item->FrameStart;
	cd_comnd_respond(COMND_TOCVAL_STRTMINS | (unsigned int)(start / MAX_MINSIZE));
	start %= MAX_MINSIZE;
	cd_comnd_respond(COMND_TOCVAL_STRTSECS | (unsigned int)(start / MAX_SECSIZE));
	start %= MAX_SECSIZE;
	cd_comnd_respond(COMND_TOCVAL_STRTFRMS | (unsigned int)(start));
	return;
}

static	void	cd_comnd_pause(unsigned int param)		/* 04h */
{
	pause_mode = TRUE;			/* CD-module goes into pause mode if tracking */
	pause_indicator = TRUE;			/* set the pause mode indicator */
	cd_comnd_delay(cd_pause_toready);	/* incorporate the delay here */
	cd_comnd_respond(COMND_FOUND);		/* send the FOUND command */
	return;
}

static	void	cd_comnd_unpause(unsigned int param)		/* 05h */
{
	pause_mode = FALSE;			/* releases the pause mode if pausing */
	pause_indicator = FALSE;		/* clear the pause mode indicator */
	cd_comnd_delay(cd_unpause_toready);	/* incorporate the delay here */
	cd_comnd_respond(COMND_FOUND);		/* send the FOUND command */
	return;
}

static	void	cd_comnd_gettitlelen(unsigned int param)	/* 09h */
{
register ITEM	*item;
register int	sessno, trakno;
register unsigned int	length;

	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	sessno = -1, trakno = 0;		/* look for the requested track */
	for (item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if ((item->Settings & IT_SESSION) && ((++sessno) > 0)) break;
		if ((item->Settings & IT_TRACK) && ((++trakno) == param)) break;
	}
	if ((sessno > 0) || (item == NULL))
	{
		/* error if not in first session or track does not exist */
		cd_comnd_senderror(COMND_E_ILLVALUE);
		return;
	}

	/* find out the length of the track in seconds */
	length = (unsigned int)(item->FrameLength / MAX_SECSIZE);

	/* send out the track length lsb/msb with different commands */
	cd_comnd_respond(COMND_TRACKLEN_LSB | (length & 0x00FF));
	cd_comnd_respond(COMND_TRACKLEN_MSB | ((length >> 8) & 0x00FF));
	return;
}

static	void	cd_comnd_getcomptime(unsigned int param)	/* 0Dh */
{
register unsigned long	curpos;

	/* returns the current absolute time in minutes:seconds:frames */
	curpos = (curr_absqpos / MAX_FRMSIZE);
	cd_comnd_respond(COMND_ABSOLUTE_MINS | (unsigned int)(curpos / MAX_MINSIZE));
	curpos %= MAX_MINSIZE;
	cd_comnd_respond(COMND_ABSOLUTE_SECS | (unsigned int)(curpos / MAX_SECSIZE));
	curpos %= MAX_SECSIZE;
	cd_comnd_respond(COMND_ABSOLUTE_FRMS | (unsigned int)(curpos));
	return;
}

static	void	cd_comnd_gotoabsfrm(unsigned int param)		/* 12h */
{
register unsigned long	cur_frms;
register unsigned long	min_frms, max_frms;

	cd_comnd_playreset();			/* reset the current play stuff */
	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	goto_frms = param;

	/* check for validity of the given time */
	/* specified time must be greater then or equal to 0:2:0 */
	/* and less than or equal to the last leadout starting time */
	cur_frms =	((long)goto_mins * MAX_MINSIZE) +
			((long)goto_secs * MAX_SECSIZE) +
			((long)goto_frms);

	min_frms = gl_actdocu->FirstItem->FrameStart;
	max_frms = gl_actdocu->LastItem->FrameStart;
	if ((cur_frms < min_frms) || (cur_frms > max_frms))
	{					/* error if position out of bound */
		cd_comnd_senderror(COMND_E_ILLVALUE);
		return;
	}
	play_frframes = cur_frms;		/* set the play start position */
	cd_comnd_seek();			/* seek to the specified track */
	cd_comnd_setlast(0);			/* set the play end position */
	cd_comnd_playset();			/* set the current play stuff */
	cd_comnd_respond(COMND_FOUND);		/* respond depending upon the ATTI settings */
	return;
}

static	void	cd_comnd_readlongtoc(unsigned int param)	/* 14h */
{
register ITEM	*item;
register ITEM	*start_item;
register int	sessno, trakno;
register int	start_trakno;
register unsigned long	start;

	cd_comnd_playreset();			/* reset the current play stuff */
	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	sessno = -1, trakno = 0;		/* look for specified session */
	for (item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if ((item->Settings & IT_SESSION) && ((++sessno) == param)) break;
		if (item->Settings & IT_TRACK) trakno ++;
	}
	if (item == NULL)
	{
		/* return error if specified session number does not exist */
		cd_comnd_senderror(COMND_E_ILLVALUE);
		return;
	}
	play_frframes = item->FrameStart;	/* set the play start position */
	cd_comnd_seek();			/* seek to specified session */
	cd_comnd_setlast(0);			/* set the play end position */

	start_trakno = trakno;
	start_item = item;

	trakno = start_trakno;
	for (item = start_item->next ; item ; item = item->next)
	{
		if (item->Settings & (IT_SESSION | IT_LAST)) break;
		if (!(item->Settings & IT_TRACK)) continue;

		trakno ++;
		cd_comnd_respond(COMND_LONGTOC_TRACKNO  | trakno);
		cd_comnd_respond(COMND_LONGTOC_ATTRS    | 0x0000);
		start = item->FrameStart;
		cd_comnd_respond(COMND_LONGTOC_STRTMINS | (unsigned int)(start / MAX_MINSIZE));
		start %= MAX_MINSIZE;
		cd_comnd_respond(COMND_LONGTOC_STRTSECS | (unsigned int)(start / MAX_SECSIZE));
		start %= MAX_SECSIZE;
		cd_comnd_respond(COMND_LONGTOC_STRTFRMS | (unsigned int)(start));
	}
	return;
}

static	void	cd_comnd_geterror(unsigned int param)		/* 16h */
{
	cd_comnd_senderror(curr_error);
	return;
}

static	void	cd_comnd_clearerror(unsigned int param)		/* 17h */
{
	cd_comnd_senderror(COMND_E_NOERROR);
	return;
}

static	void	cd_comnd_spinup(unsigned int param)		/* 18h */
{
register ITEM	*item;
register int	sessno;

	cd_comnd_playreset();			/* reset the current play stuff */
	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	sessno = -1;				/* look for the specified session */
	for (item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if ((item->Settings & IT_SESSION) && ((++sessno) == param)) break;
	}
	if (item == NULL)
	{
		/* return error if the specified session does not exist */
		cd_comnd_senderror(COMND_E_ILLVALUE);
		return;
	}
	item = item->next;			/* track in this session */
	play_frframes = item->FrameStart;	/* set the play start position */
	cd_comnd_seek();			/* seek to the specified track */
	cd_comnd_setlast(0);			/* set the play end position */
	cd_comnd_playset();			/* set the current play stuff */
	pause_mode = TRUE;			/* now the disc goes into pause mode */
	cd_comnd_respond(COMND_FOUND);		/* send the found command */
	return;
}

static	void	cd_comnd_plabstopfrm(unsigned int param)	/* 25h */
{
unsigned long	atime, btime;

	cd_comnd_playreset();			/* reset the current play stuff */
	cd_comnd_discspin(TRUE);		/* spin the disc if not already spun */
	abplay_bfrms = param;

	/* check for validity of the start and end time */
	/* A-time and B-time must be in one program area */
	/* B-time must be greater than or equal to A-time */

	atime =	((long)abplay_amins * MAX_MINSIZE) +
		((long)abplay_asecs * MAX_SECSIZE) +
		((long)abplay_afrms);

	btime =	((long)abplay_bmins * MAX_MINSIZE) +
		((long)abplay_bsecs * MAX_SECSIZE) +
		((long)abplay_bfrms);

	play_frframes = atime;			/* set the play start position */
	cd_comnd_seek();			/* seek to the specified pos */
	cd_comnd_setlast(btime);		/* set the play end position */
	cd_comnd_playset();			/* set the current play stuff */
	cd_comnd_respond(COMND_FOUND);
	return;
}

static	void	cd_comnd_abrelease(unsigned int param)		/* 26h */
{
	cd_comnd_setlast(0);			/* set the last q-time position */
	cd_comnd_respond(COMND_A2BRELEASED);	/* send the A->B released command */
	return;
}

static	void	cd_comnd_getmaxsession(unsigned int param)	/* 54h */
{
	cd_comnd_respond(COMND_SESSIONINFO | gl_actdocu->NoSessions);
	return;
}

/******************************************************************************/
/******************************************************************************/

static	void	cd_comnd_senderror(unsigned int errorval)
{
	curr_error = errorval;
	cd_comnd_respond(COMND_ERRORVALUES | errorval);
	return;
}

static	void	cd_comnd_respond(unsigned int command)
{
	DSA_out = command;
	TX_DSA();
	return;
}

/******************************************************************************/
/******************************************************************************/

#define	FACTOR	(1000 / CLK_TCK)
void	cd_comnd_delay(delay)
register unsigned int	delay;
{
register long	*clockptr;
register long	stopat;

	if (!delay)
		return;

	clockptr = (long *)(HZ_200);
	stopat = (*clockptr) + (delay / FACTOR);
	while ((*clockptr) < stopat);
	return;
}
#undef	FACTOR

static	void	cd_comnd_seekdelay(oldpos, newpos)
register unsigned long	oldpos;
register unsigned long	newpos;
{
register long	diff = ((oldpos < newpos) ? (newpos - oldpos) : (oldpos - newpos));
register long	delay;

	play_seektime = 0l;
	if (diff == 0)
		return;

	if (diff > MAX_MINSIZE)
	{
		/* implement long seek delay here */
		     if (oldpos < (20l * MAX_MINSIZE))	delay = cd_longseek_00to20;
		else if (oldpos < (40l * MAX_MINSIZE))	delay = cd_longseek_21to40;
		else if (oldpos < (60l * MAX_MINSIZE))	delay = cd_longseek_41to60;
		else if (oldpos < (75l * MAX_MINSIZE))	delay = cd_longseek_61to74;
	}
	else
	{
		/* implement short seek delay here */
		     if (oldpos < (37l * MAX_MINSIZE))	delay = cd_shortseek_1sthalf;
		else if (oldpos < (75l * MAX_MINSIZE))	delay = cd_shortseek_2ndhalf;
	}
	play_seektime = delay;

/*==============================================*/
#if SEEKBUFF > 0
/*==============================================*/
	if (!cd_preload_buffsize)
/*==============================================*/
#endif
/*==============================================*/
	{
		cd_comnd_delay(delay);
	}
	return;
}

/******************************************************************************/
/******************************************************************************/

static	void	cd_comnd_discspin(int which)
{
register CDROM	*docu = gl_actdocu;
register unsigned int	NoSessions = docu->NoSessions;
register unsigned int	NoTracks = docu->NoTracks;
register unsigned int	delay;

	if (which)
	{
		if (disc_spinning)
			return;

		delay = cd_spinup;
		if (NoSessions > 1) delay += (NoSessions - 1) * cd_spinup_addsession;
		if (NoTracks > 10) delay += (NoTracks - 10) * cd_spinup_addtrack;
		disc_spinning = TRUE;
	}
	else
	{
		if (disc_spinning)
			return;

		delay = cd_stopdisc;
		disc_spinning = FALSE;
	}
	cd_comnd_delay(delay);		/* insert the delay here */
	return;
}

static	void	cd_comnd_playreset()
{
	play_flag = PLAY_NONE;		/* reset the flag to stop play */
	return;
}

static	void	cd_comnd_setlast(lastfrms)
unsigned long	lastfrms;
{
register ITEM	*item = gl_actdocu->LastItem;

	play_toframes = ((lastfrms > 0) ? lastfrms : (item->FrameStart + item->FrameLength));
	curr_lastpos = play_toframes * (unsigned long)MAX_FRMSIZE;
	return;
}

static	void	cd_comnd_seek()
{
register ITEM	*item, *prev;
register unsigned long	curpos, newpos;

	/* seek to the new position */
	curpos = curr_absqpos / MAX_FRMSIZE;
	newpos = play_frframes;

	/* delay routine goes here */
	cd_comnd_seekdelay(curpos, newpos);

	/* set the current absolute Q-time */
	curr_absqpos = play_frframes * (unsigned long)MAX_FRMSIZE;

	/* find out the session/track/item for the newposition */
	prev = NULL;
	for (item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if (item->FrameStart  > newpos)	break;
		prev = item;
		if (item->FrameStart == newpos)	break;
	}
	curr_item = prev;
	return;
}

static	void	cd_comnd_playset()
{
register unsigned int	delay;

	/* if internal pause mode indicator is on, then */
	/* CD-Module goes into pause mode */
	if (pause_indicator) pause_mode = TRUE;

	play_flag = PLAY_NONE;		/* re-initialise play_flag */
	item_padding = 0;		/* length of file padding */

	/* initialise the DSP buffer */
	buff_length = fsel_initbuff();

	/* set different parameters to initiate disk read */
/*==============================================*/
#if SEEKBUFF > 0
/*==============================================*/
	if (!cd_comnd_seekread())
/*==============================================*/
#else
/*==============================================*/
	if (!cd_comnd_setread())
/*==============================================*/
#endif
/*==============================================*/
	{
		cd_comnd_senderror(COMND_E_FOCUSORNODISC);
		play_flag = PLAY_NONE;
		return;
	}

	for (delay = 0x4000 ; delay ; delay--);

	/* set the DSP to accept data at host port */
	BuffIni();
	HostIni();
	return;
}

/*==============================================*/
#if SEEKBUFF > 0
/*==============================================*/

static	int	cd_comnd_seekread()
{
register ITEM	*item = curr_item;
register unsigned long	absqpos = curr_absqpos;
register unsigned long	endpos;
register unsigned int	type;
register long	currpos;
register long	currlen;
register long	addpos;

	/* set the limit of the seek buffer */
	endpos = (play_seektime * cd_preload_readrate) / 1000;
	if (endpos > cd_preload_buffsize) endpos = cd_preload_buffsize;

/*==============================================*/
#if ONEREADS > 0
/*==============================================*/

	if (endpos < 0x10000) endpos = 0x10000l;

/*==============================================*/
#endif
/*==============================================*/

	BuffSet(endpos);

	if (!C_limit)
	{
		/* set parameters to initiate disk read */
		return(cd_comnd_setread());
	}
	while (C_ldind < C_limit)
	{
		/* get the type of the current item */
		type = ((item == NULL) ? 0 : (item->Settings & IT_TYPE));
		if (type == IT_LAST)
			break;

		/* the position in current item where the data flow will start */
		currpos = absqpos - (type ? (item->FrameStart * MAX_FRMSIZE) : 0);

		if (type == IT_ITEM)
		{
			currlen = item->Length;
			if (currlen % FSEL_BUFFSIZE) currlen += FSEL_BUFFSIZE;
			currlen = (currlen / FSEL_BUFFSIZE) * FSEL_BUFFSIZE;

			if (currpos < currlen)
			{
				addpos = fsel_seekreadfile(item->Cluster, currpos, currlen);
				if (addpos < 0)
				{
					return(FALSE);
				}
				absqpos += addpos;
				if (item == curr_item) C_rdind = (currpos % FSEL_BUFFSIZE);
				currpos = 0;
			}
			else
			{
				absqpos += currlen;
				currpos -= currlen;
			}
			currlen = (item->FrameLength * MAX_FRMSIZE) - currlen;

			type = (item->next->Settings & IT_TYPE);
			if (type == IT_TRACK)
			{
				currlen += cd_traktail * (long)MAX_FRMSIZE;
			}
			if ((type == IT_SESSION) || (type == IT_LAST))
			{
				currlen += cd_traktail * (long)MAX_FRMSIZE;
				currlen += cd_sesstail * (long)MAX_FRMSIZE;
			}
		}
		else if (type == IT_TRACK)
			currlen = cd_traklead * (long)MAX_FRMSIZE;
		else if (type == IT_SESSION)
			currlen = cd_sesslead * (long)MAX_FRMSIZE;
		else if (type == IT_LAST)
			currlen = cd_cdrmtail * (long)MAX_FRMSIZE;
		else	currlen = cd_cdrmlead * (long)MAX_FRMSIZE;

		endpos = absqpos + (currlen - currpos);
		addpos = fsel_seekreadzero(currpos, currlen);
		absqpos += addpos;
		if (absqpos < endpos)
			break;

		item = item->next;
	}
	if (C_ldind > C_rdind)
	{
		curr_item = item;
		play_flag = PLAY_BUFF;
	}
	return(TRUE);
}

/*==============================================*/
#endif
/*==============================================*/

static	int	cd_comnd_setread()
{
register ITEM	*item;
register unsigned int	type;
register long	currpos;
register long	strtpos;
register long	currlen;
register long	paddlen;
register int	err;

	/* do stuff about the new item and buffer status etc */
	/* seek to the new position */
	if ((item = curr_item) != NULL)
	{
		strtpos = (item->FrameStart * MAX_FRMSIZE);
		type = (item->Settings & IT_TYPE);
	}
	else
	{
		strtpos = 0l;
		type = 0;
	}
	switch(type)
	{
	case IT_ITEM :
		play_flag = PLAY_FILE;
		currlen = item->Length;
		if (currlen % FSEL_BUFFSIZE) currlen += FSEL_BUFFSIZE;
		currlen /= FSEL_BUFFSIZE;
		currlen *= FSEL_BUFFSIZE;
		paddlen = (item->FrameLength * MAX_FRMSIZE) - currlen;

		type = (item->next->Settings & IT_TYPE);
		if (type == IT_ITEM)
			break;

		paddlen += cd_traktail * (long)MAX_FRMSIZE;
		if (type == IT_TRACK)
			break;

		paddlen += cd_sesstail * (long)MAX_FRMSIZE;
		break;

	case IT_TRACK :
		play_flag = PLAY_ZERO;
		currlen = cd_traklead * (long)MAX_FRMSIZE;
		paddlen = 0;
		break;

	case IT_SESSION :
		play_flag = PLAY_ZERO;
		currlen = cd_sesslead * (long)MAX_FRMSIZE;
		paddlen = 0;
		break;

	case IT_LAST :
		play_flag = PLAY_ZERO;
		currlen = cd_cdrmtail * (long)MAX_FRMSIZE;
		paddlen = 0;
		break;

	default :
		play_flag = PLAY_ZERO;
		currlen = cd_cdrmlead * (long)MAX_FRMSIZE;
		paddlen = 0;
		break;
	}
	/* offset of the current requested position wrt the */
	/* start of current data file or zero data buffer */
	currpos = curr_absqpos - strtpos;
	if ((play_flag == PLAY_FILE) && (currpos >= currlen))
	{
		play_flag = PLAY_ZERO;
		currpos -= currlen;
		strtpos += currlen;
		currlen = paddlen;
		paddlen = 0;
	}
	item_endpos = strtpos + currlen;
	item_padding = paddlen;

	/* initialise the DSP buffer and hard drive read routines */
	if (play_flag == PLAY_FILE)
		err = fsel_initfiledata(item->Cluster, currpos, currlen);
	else	err = fsel_initzerodata(currpos, currlen);
	return((err <= 0) ? FALSE : TRUE);
}

static	void	cd_comnd_readdata()
{
register ITEM	*item;
register int	addpos;

/*==============================================*/
#if SEEKBUFF > 0
/*==============================================*/

	if (C_rdind < C_ldind)
	{
		/* send data out from seek buffer */
		addpos = fsel_seeksenddata();

		/* modify the current CDROM head position */
		curr_absqpos += (long)addpos;
		if (C_rdind < C_ldind)
			return;

		/* the seek buffer is empty now */
		/* set parameters to initiate periodic disk read */
		if (!cd_comnd_setread())
		{
			play_flag = PLAY_NONE;
			return;
		}
	}

/*==============================================*/
#endif
/*==============================================*/

	for (;;)
	{
		addpos = (play_flag == PLAY_FILE) ? fsel_readfiledata() : fsel_readzerodata();
		if (addpos < 0)
			break;

		/* modify the current CDROM head position */
		curr_absqpos += (long)addpos;

		/* return if lot more to be loaded from the file or zero buffer */
		if (curr_absqpos < item_endpos)
			return;

		/* if the current file is loaded then */
		/* move on to the next */
		if ((play_flag == PLAY_FILE) && (item_padding > 0))
		{
			play_flag = PLAY_ZERO;
			item_endpos += item_padding;
			fsel_initzerodata(0l, item_padding);
			item_padding = 0;
			continue;
		}
		item = ((item = curr_item) ? item->next : gl_actdocu->FirstItem);
		if (curr_item = item)
		{
			if (cd_comnd_setread())
				continue;
			break;
		}
		pause_mode = TRUE;
	}
	play_flag = PLAY_NONE;
	return;
}

/******************************************************************************/
/* End of CD_COMND.C							      */
/******************************************************************************/
