/******************************************************************************/
/* ST_FILE.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"

#include "st_aes.h"
#include "st_bind.h"
#include "st_lib.h"
#include "cd_bind.h"

/* -------------------------------------------------------------------------- */

extern	unsigned char	*gl_untitledstr;

extern	CDROM	*gl_actdocu;

extern	int	cd_sesshite;	/* hite of each session row */
extern	int	cd_trakhite;	/* hite of each track row */
extern	int	cd_itemhite;	/* hite of each item row */
extern	int	cd_lasthite;	/* hite of the end of rom row */

extern	int	cd_sessoffs;	/* distance of bottom of row to base line in session row */
extern	int	cd_trakoffs;	/* distance of bottom of row to base line in track row */
extern	int	cd_itemoffs;	/* distance of bottom of row to base line in item row */
extern	int	cd_lastoffs;	/* distance of bottom of row to base line in last row */

extern	int	cd_sessstartx;	/* starting x position of session */
extern	int	cd_trakstartx;	/* starting x position of track */
extern	int	cd_itemstartx;	/* starting x position of item */
extern	int	cd_laststartx;	/* starting x position of the end of CDROM row */

/* -------------------------------------------------------------------------- */

#define	ST_VER_MAJOR	'1'	/* major version number */
#define	ST_VER_MINOR	'2'	/* minor version number */

/* -------------------------------------------------------------------------- */

static	int	cd_clipfile = FALSE;

/* -------------------------------------------------------------------------- */

static	int	cd_save_item( ITEM *, FILE * );
static	int	cd_load_items( CDROM *, FILE * );

static	int	cd_load_update( CDROM * );

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Frees the contents of the given document
 */

void	cd_free(docu)
register CDROM	*docu;
{
register ITEM	*item;

	if (docu == NULL)
		return;

	for (item = docu->FirstItem ; item ; item = item->next)
	{
		free(item);
	}
	free(docu);
	free_arenas();
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Create a new document
 */

CDROM	*cd_create()
{
register CDROM	*docu;
register ITEM	*item;

	docu = (CDROM *)calloc((size_t)1, (size_t)sizeof(CDROM));
	if (docu == NULL)
	{
		cd_free(docu);
		return(NULL);
	}
	docu->Settings |= GL_DOCUNEW;
	strcpy(docu->CdRomName, gl_untitledstr);
	strcpy(docu->PathName, gl_untitledstr);

	item = cd_create_item(IT_LAST);
	if (item == NULL)
	{
		cd_free(docu);
		return(NULL);
	}
	docu->FirstItem =
	docu->LastItem =
	docu->SelItem = item;

	docu->NoSessions = 0;
	docu->NoTracks = 0;
	docu->NoItems = 0;
	return(docu);
}

/* -----------------------------------------------------------------------------
 */

ITEM	*cd_create_item(settings)
register unsigned int	settings;
{
register ITEM	*item;

	item = (ITEM *)calloc((size_t)1, (size_t)sizeof(ITEM));
	if (item == NULL)
	{
		return(NULL);
	}
	if (settings & IT_LAST)
	{
		item->Settings |= IT_LAST;
		item->Hite	= cd_lasthite;
		item->OffsetY	= cd_lastoffs;
		item->StartX	= cd_laststartx;
	}
	else
	if (settings & IT_SESSION)
	{
		item->Settings |= IT_SESSION;
		item->Hite	= cd_sesshite;
		item->OffsetY	= cd_sessoffs;
		item->StartX	= cd_sessstartx;
	}
	else
	if (settings & IT_TRACK)
	{
		item->Settings |= IT_TRACK;
		item->Hite	= cd_trakhite;
		item->OffsetY	= cd_trakoffs;
		item->StartX	= cd_trakstartx;
	}
	else
	if (settings & IT_ITEM)
	{
		item->Settings |= IT_ITEM;
		item->Hite	= cd_itemhite;
		item->OffsetY	= cd_itemoffs;
		item->StartX	= cd_itemstartx;

		strcpy(item->PathName, gl_untitledstr);
	}
	return(item);
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Save the global information like document name, document type resolution
 * and page setup information at the begining of each File
 */

int	cd_save(docu, fp)
register CDROM	*docu;
register FILE	*fp;
{
register ITEM	*item;
register int	sessno;
register int	trakno;
register int	itemno;
unsigned char	header[30];

/* get the number of sessions/tracks/items to be saved */

	sessno = trakno = itemno = 0;
	for (item = docu->FirstItem ; item != docu->LastItem ; item = item->next)
	{
		if (cd_clipfile && (!(item->Settings & IT_SELECTED)))
			continue;

		if (item->Settings & IT_SESSION)	sessno ++;
		if (item->Settings & IT_TRACK)		trakno ++;
		if (item->Settings & IT_ITEM)		itemno ++;
	}

/* prepare the global document header */

	header[0] = ST_VER_MAJOR;				/* save the major version number */
	header[1] = ST_VER_MINOR;				/* save the minor version number */
	header[2] = 'C';					/* magic number for */
	header[3] = 'D';					/* CD ROM emulator */
	header[4] = 16;						/* header length */
	header[5] = (unsigned char)strlen(docu->CdRomName);	/* length of CD ROM name */
	header[6] = 0;						/* reserved */
	header[7] = 0;						/* reserved */
	(*((unsigned int *)(&header[ 8]))) = docu->Settings;	/* document settings */
	(*((unsigned int *)(&header[10]))) = sessno;		/* number of sessions in CDROM */
	(*((unsigned int *)(&header[12]))) = trakno;		/* numbers of tracks in CDROM */ 
	(*((unsigned int *)(&header[14]))) = itemno;		/* number of files in CDROM */

	if (!fwrite(header, (size_t)1, (size_t)16, fp))
		return(FALSE);

/* save CD ROM name as specified by the user */

	if (header[5] && !fwrite(docu->CdRomName, (size_t)1, (size_t)header[5], fp))
		return(FALSE);

/* save sessions */

	for (item = docu->FirstItem ; item != docu->LastItem ; item = item->next)
	{
		if (cd_clipfile && (!(item->Settings & IT_SELECTED)))
			continue;

		if (!cd_save_item(item, fp))
			return(FALSE);
	}
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 * Loads the global information from the begining of a file
 */

CDROM	*cd_load(fp, ret)
register FILE	*fp;
register int	*ret;
{
register CDROM	*docu;
register unsigned int	Settings;
register unsigned int	NoSessions;
register unsigned int	NoTracks;
register unsigned int	NoItems;
unsigned char	header[30];

/* read the global document header and update stuff */

	(*ret) = 2;
	if (!fread(header, (size_t)1, (size_t)16, fp))
		return(NULL);

/* header[0] and header[1] give the version number */
/* header[2] and header[3] give the magic number */

	(*ret) = 3;
	if ((header[0] != ST_VER_MAJOR) || (header[1] != ST_VER_MINOR))
		return(NULL);

	if ((header[2] != 'C') || (header[3] != 'D'))
		return(NULL);

/* header[4] gives the length of header */

	(*ret) = 2;
	if (header[4] != 16)
		return(NULL);

/* header[5] gives the length of CD ROM name */

	if (header[5] >= MAX_ROMNAME)
		return(NULL);

/* header[6] and header[7] are reserved */

	if ((header[6] != 0) || (header[7] != 0))
		return(NULL);

/* header[ 8] and header[ 9] give the document settings */
/* header[10] and header[11] give the number of sessions in the CDROM */
/* header[12] and header[13] give the number of tracks in CDROM */
/* header[14] and header[15] give the number of files in CDROM */

	Settings = (*((unsigned int *)(&header[ 8])));
	NoSessions = (*((unsigned int *)(&header[10])));
	NoTracks = (*((unsigned int *)(&header[12])));
	NoItems = (*((unsigned int *)(&header[14])));

	if ((NoSessions > MAX_SESSIONS) || (NoTracks > MAX_TRACKS) || (NoItems > MAX_ITEMS))
		return(NULL);

/* create the CDROM header */

	(*ret) = 1;
	docu = cd_create();
	if (docu == NULL)
		return(NULL);

	docu->Settings = Settings;
	docu->NoSessions = NoSessions;
	docu->NoTracks = NoTracks;
	docu->NoItems = NoItems;

/* read the CD ROM name */

	(*ret) = 2;
	if (header[5] && !fread(docu->CdRomName, (size_t)1, (size_t)header[5], fp))
		return(docu);

	docu->CdRomName[ header[5] ] = '\0';

	docu->Settings &= ~(GL_DOCUNEW | GL_DOCUNOTSAVED);

/* read sessions */

	(*ret) = cd_load_items(docu, fp);
	if (*ret)
	{
		return(docu);
	}

/* update the item list so that they are in the proper order of session/track/file(s) */
/* this is mainly essential for loading a document which was copied to the clipboard */
/* since clipboard document can start anywhere - like from file or track also */

	if (!cd_clipfile)
	{
		(*ret) = cd_load_update(docu);
		if (*ret)
		{
			return(docu);
		}
	}

/* set selection parameters - if load successful */

	docu->SelItem = docu->FirstItem;
	return(docu);
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

int	cd_clipexport(fp)
FILE	*fp;
{
register CDROM	*docu = gl_actdocu;
register ITEM	*item;
register unsigned int	settings;
unsigned char	messstr[300];
unsigned char	ctrlstr[300];

	sprintf(messstr, "CDROM : %s\n", docu->CdRomName);
	if (!fwrite(messstr, (size_t)1, (size_t)strlen(messstr), fp))
		return(FALSE);

	for (item = docu->FirstItem ; item ; item = item->next)
	{
		settings = item->Settings;
		if (!(settings & IT_SELECTED))
			continue;

		strcpy(messstr, "");
		if (settings & IT_TRACK)	strcat(messstr, "  ");
		if (settings & IT_ITEM)		strcat(messstr, "  ");

		cd_y_coordinate(item);
		cd_display_str(item, ctrlstr);
		strcat(messstr, ctrlstr);

		if (settings & (IT_TRACK | IT_ITEM))
		{
			sprintf(ctrlstr, " [ %ld ] [ ", item->Length);
			strcat(messstr, ctrlstr);
			cd_display_framestr(item->FrameStart, ctrlstr);
			strcat(messstr, ctrlstr);
			strcat(messstr, " ] [ ");
			cd_display_framestr(item->FrameLength, ctrlstr);
			strcat(messstr, ctrlstr);
			strcat(messstr, " ] ");
			strcat(messstr, ctrlstr);
		}
		sprintf(ctrlstr, "[ %s ]\n", item->Comments);
		strcat(messstr, ctrlstr);

		if (!fwrite(messstr, (size_t)1, (size_t)strlen(messstr), fp))
			return(FALSE);
	}
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 */

int	cd_clipsave(fp)
FILE	*fp;
{
register int	ret;

	cd_clipfile = TRUE;
	ret = cd_save(gl_actdocu, fp);
	cd_clipfile = FALSE;
	return(ret);
}

/* -----------------------------------------------------------------------------
 */

CDROM	*cd_clipload(fp, ret)
FILE	*fp;
int	*ret;
{
register CDROM	*docu;

	cd_clipfile = TRUE;
	docu = cd_load(fp, ret);
	cd_clipfile = FALSE;
	return(docu);
}

/* -----------------------------------------------------------------------------
 */

void	cd_clipfree(docu)
CDROM	*docu;
{
	cd_free(docu);
	return;
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	int	cd_save_item(item, fp)
register ITEM	*item;
register FILE	*fp;
{
unsigned char	header[30];

	(*((unsigned int *)(&header[0]))) = item->Settings;
	(*((unsigned long *)(&header[2]))) = item->Length;

	header[6] = strlen(item->Comments);
	header[7] = strlen(item->PathName);

	(*((unsigned long *)(&header[8]))) = item->FrameStart;
	(*((unsigned long *)(&header[12]))) = item->FrameLength;

	if (!fwrite(header, (size_t)1, (size_t)16, fp))
		return(FALSE);

	if (header[6] && !fwrite(item->Comments, (size_t)1, (size_t)header[6], fp))
		return(FALSE);

	if (header[7] && !fwrite(item->PathName, (size_t)1, (size_t)header[7], fp))
		return(FALSE);

	return(TRUE);
}

/* -----------------------------------------------------------------------------
 */

static	int	cd_load_items(docu, fp)
register CDROM	*docu;
register FILE	*fp;
{
register ITEM	*next;
register ITEM	*item;
register unsigned int	i;
register unsigned int	TotalNo;
register unsigned int	settings;
unsigned char	header[30];

	TotalNo = docu->NoSessions + docu->NoTracks + docu->NoItems;

	next = docu->LastItem;
	for (i = 0 ; i < TotalNo ; i++)
	{
		if (!fread(header, (size_t)1, (size_t)16, fp))
			return(2);

		settings = (*((unsigned int *)(&header[0])));
		item = cd_create_item(settings);
		if (item == NULL)
			return(1);

		if (item->prev = next->prev)
			item->prev->next = item;
		else	docu->FirstItem = item;
		item->next = next;
		next->prev = item;

		item->Settings	= settings;
		item->Length	= (*((unsigned long *)(&header[2])));

		if ((header[6] >= MAX_COMLEN) || (header[7] >= MAX_PATHNAME))
			return(2);

		item->FrameStart  = (*((unsigned long *)(&header[8])));
		item->FrameLength = (*((unsigned long *)(&header[12])));

		if (header[6] && !fread(item->Comments, (size_t)1, (size_t)header[6], fp))
			return(2);

		if (header[7] && !fread(item->PathName, (size_t)1, (size_t)header[7], fp))
			return(2);

		item->Comments[ header[6] ] = '\0';
		item->PathName[ header[7] ] = '\0';

		item->Settings &= ~IT_SELECTED;
	}
	return(0);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	int	cd_load_update(docu)
register CDROM	*docu;
{
register ITEM	*item;
register ITEM	*newitem;
register unsigned int	cur_type;
register unsigned int	req_type;
register unsigned int	settings;

	req_type = (IT_SESSION | IT_LAST);
	for (item = docu->FirstItem ; item ; item = item->next)
	{
		cur_type = (item->Settings & IT_TYPE);

		if (!(req_type & cur_type))
		{
			cur_type = (req_type & (IT_SESSION | IT_TRACK | IT_ITEM));

			newitem = cd_create_item(cur_type);
			if (newitem == NULL)
			{
				return(1);
			}
			if (newitem->prev = item->prev)
				newitem->prev->next = newitem;
			else	docu->FirstItem = newitem;
			newitem->next = item;
			item->prev = newitem;

			settings = newitem->Settings;
			if (settings & IT_ITEM)		docu->NoItems ++;
			if (settings & IT_TRACK)	docu->NoTracks ++;
			if (settings & IT_SESSION)	docu->NoSessions ++;
			item = newitem;
		}

		if (cur_type & IT_SESSION)	req_type = IT_TRACK;
		if (cur_type & IT_TRACK)	req_type = IT_ITEM;
		if (cur_type & IT_ITEM)		req_type = IT_TYPE;
	}
	cd_update_time(docu);
	return(0);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_save_filelist()
{
register ITEM	*item;
register FILE	*fp;
register int	i;
register unsigned long	nofrms;
register int	mins, secs, frms;
unsigned char	filename[ MAX_PATHNAME ];

	strcpy(filename, "FILELIST.INC");
	if (!st_get_filename(2, AWRITE, filename, ".INC", FSEL_ITEMFILENAME))
		return;

	st_mouse_busy();
	fp = fopen(filename, "wb");
	if (fp == NULL)
	{
		st_error_message(E_OPENF);
		return;
	}

	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, ";* TITLE        : %-12s                                                *\n", st_str_filename(filename));
	fprintf(fp, ";* Function     : This file contains a file table which provides the          *\n");
	fprintf(fp, ";*                position of the file on the disc and its length             *\n");
	fprintf(fp, ";*                                                                            *\n");
	fprintf(fp, ";* Project #    : JAGUAR SLIDESHOW from Sample CD                             *\n");
	fprintf(fp, ";* Programmer   : Pradip K Fatehpuria                                         *\n");
	fprintf(fp, ";*                                                                            *\n");
	fprintf(fp, ";* COPYRIGHT(c) 1992,1993 Atari Computer Corporation                          *\n");
	fprintf(fp, ";* UNATHORIZED REPRODUCTION, ADAPTATION, DISTRIBUTION, PERFORMANCE OR         *\n");
	fprintf(fp, ";* DISPLAY OF THIS COMPUTER PROGRAM OR THE ASSOCIATED AUDIOVISUAL WORK        *\n");
	fprintf(fp, ";* IS STRICTLY PROHIBITED.  ALL RIGHTS RESERVED.                              *\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, "\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, ";*      EQUATES                                                               *\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, "\n");
	fprintf(fp, "MAXFILES\tequ\t%d\n", gl_actdocu->NoItems);
	fprintf(fp, "\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, ";*      CONSTANT DATA AREA                                                    *\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, "\n");
	fprintf(fp, "\t.data\n");
	fprintf(fp, "\n");
	fprintf(fp, ";*============================================================================*\n");
	fprintf(fp, ";*============================================================================*\n");
	fprintf(fp, "\n");
	fprintf(fp, "filtable:\t.phrase\n", filename);
	for (i = 0, item = gl_actdocu->FirstItem ; item ; item = item->next)
	{
		if (!(item->Settings & IT_ITEM))
			continue;

		nofrms = item->FrameStart;
		mins = (int)(nofrms / MAX_MINSIZE);
		nofrms %= MAX_MINSIZE;
		secs = (int)(nofrms / MAX_SECSIZE);
		nofrms %= MAX_SECSIZE;
		frms = (int)(nofrms);

		fprintf(fp, "\t\t.dc.l\t$00%02x%02x%02x,$%08lx", mins, secs, frms, item->Length);
		fprintf(fp, "\t; %3d %-12s -", ++i, st_str_filename(item->PathName));

		nofrms = item->FrameStart;
		if (nofrms >= 6) nofrms -= 6;
		mins = (int)(nofrms / MAX_MINSIZE);
		nofrms %= MAX_MINSIZE;
		secs = (int)(nofrms / MAX_SECSIZE);
		nofrms %= MAX_SECSIZE;
		frms = (int)(nofrms);

		fprintf(fp, " ($00%02x%02x%02x) (%8ld)\n", mins, secs, frms, item->Length);
	}
	fprintf(fp, "\t\t.phrase\n");
	fprintf(fp, "\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, ";*      END OF FILE                                                           *\n");
	fprintf(fp, ";******************************************************************************\n");
	fprintf(fp, "\n");
	fprintf(fp, "\t.end\n");

	fclose(fp);
	st_mint_send_winddraw(filename);
	return;
}

/******************************************************************************/
/* End of ST_FILE.C							      */
/******************************************************************************/
