/******************************************************************************/
/* CD_INIT.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"
#include "st_aes.h"
#include "st_bind.h"
#include "st_lib.h"
#include "st_scrap.h"
#include "cd_bind.h"
#include "cd_disp.h"

#include "cdmonusa.h"

/* -------------------------------------------------------------------------- */

extern	int	gl_charwide;
extern	int	gl_charhite;

extern	int	gl_scrhandle;
extern	int	gl_nocolors;

extern	unsigned int	gl_sysfont;
extern	unsigned int	gl_syssize;

extern	OBJECT	*gl_actmenu;

extern	CDROM	*gl_actdocu;
extern	int	gl_acthandle;

/* -------------------------------------------------------------------------- */

extern	CDROM	*undo_gl_actdocu;

extern	int	cd_srchflag;

/* -------------------------------------------------------------------------- */

GRECT	gl_windrect;		/* total window area on the screen */

GRECT	cd_totlrect;		/* total work area in the window */
GRECT	cd_titlrect;		/* cd title area in the window */
GRECT	cd_inforect;		/* cd info area in window */
GRECT	cd_workrect;		/* cd header+datarect area in the window */
GRECT	cd_headrect;		/* cd headers for the track table */
GRECT	cd_datarect;		/* cd sessions/tracks area in the window */

/* CD ROM screen parameters for screen headers */

int	cd_titlhite;		/* hite of title bar in the window */
int	cd_titloffs;		/* distance of bottom of row to base line in title row */

int	cd_infohite;		/* hite of info bar in the window */
int	cd_infooffs;		/* distance of bottom of row to base line in info row */

/* CD ROM Authoring screen parameters */

int	cd_headhite;		/* hite of column headers in the window */
int	cd_headoffs;		/* distance of bottom of row to base line in header row */

int	cd_sesshite;		/* hite of each session row */
int	cd_sessoffs;		/* distance of bottom of row to base line in session row */
int	cd_sessstartx;		/* starting x position of session */

int	cd_trakhite;		/* hite of each track row */
int	cd_trakoffs;		/* distance of bottom of row to base line in track row */
int	cd_trakstartx;		/* starting x position of track */

int	cd_itemhite;		/* hite of each item row */
int	cd_itemoffs;		/* distance of bottom of row to base line in item row */
int	cd_itemstartx;		/* starting x position of item */

int	cd_lasthite;		/* hite of the end of rom row */
int	cd_lastoffs;		/* distance of bottom of row to base line in last row */
int	cd_laststartx;		/* starting x position of the end of CDROM row */

int	cd_sizestartx;		/* starting x position of size field */
int	cd_stimstartx;		/* starting x position of start time */
int	cd_etimstartx;		/* starting x position of end time */
int	cd_commstartx;		/* starting x position of comment */

/* some general display paramaters for the CD ROM Emulator screens */

long	cd_docuwide;		/* document width on the screen */
long	cd_docuhite;		/* document hite on the screen */

long	cd_visx;		/* visual x start position of the document */
long	cd_visy;		/* visual y start position of the document */

unsigned int	cd_evnlinetype = 0xAAAA; /* user defined line types */
unsigned int	cd_oddlinetype = 0x5555;

ITEM	*cd_inv_item;

/* -------------------------------------------------------------------------- */

static	void	cd_page_up( void );
static	void	cd_page_down( void );
static	void	cd_row_up( void );
static	void	cd_row_down( void );
static	void	cd_page_left( void );
static	void	cd_page_right( void );
static	void	cd_col_left( void );
static	void	cd_col_right( void );

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Global CD ROM emulator module initialisation routine
 */

int	cd_initialise()
{
register int	handle = gl_scrhandle;
int	arr[5], ad[3];
int	junk;

/* initialise VDI */

	vswr_mode(handle, MD_REPLACE);

	vsl_color(handle, BLACK);
	vsl_type(handle, 1);
	vsl_width(handle, 1);
	vsl_ends(handle, 0, 0);

	vsf_interior(handle, 1);
	vsf_perimeter(handle, 0);
	vsf_color(handle, WHITE);

	vst_rotation(handle, 0);
	vst_color(handle, DISP_DEFFCOLOR);
	vst_alignment(handle, 0, 0, &junk, &junk);
	vst_font(handle, gl_sysfont);
	vst_arbpt(handle, gl_syssize, &junk, &junk, &junk, &junk);
	vst_effects(handle, NORMAL);

	vqt_fontinfo(handle, &junk, &junk, arr, &junk, ad); 

/* initialise the screen header parameters */

	cd_titlhite = gl_charhite + 3;
	cd_titloffs = arr[0] + 2;

	cd_infohite = gl_charhite + 3;
	cd_infooffs = arr[0] + 2;

/* initialise the CD ROM Authoring screen parameters */

	cd_headhite = gl_charhite + 3;
	cd_headoffs = arr[0] + 2;

	if (gl_nocolors < 16)
	{
		cd_sesshite = gl_charhite + 10;
		cd_sessoffs = arr[0] + 6;

		cd_trakhite = gl_charhite + 8;
		cd_trakoffs = arr[0] + 5;

		cd_itemhite = gl_charhite + 4;
		cd_itemoffs = arr[0] + 3;

		cd_lasthite = gl_charhite + 6;
		cd_lastoffs = arr[0] + 4;
	}
	else
	{
		cd_sesshite = gl_charhite + 5;
		cd_sessoffs = arr[0] + 4;

		cd_trakhite = gl_charhite + 5;
		cd_trakoffs = arr[0] + 4;

		cd_itemhite = gl_charhite + 5;
		cd_itemoffs = arr[0] + 4;

		cd_lasthite = gl_charhite + 5;
		cd_lastoffs = arr[0] + 4;
	}

	cd_sessstartx = 0;
	cd_trakstartx = gl_charwide * 2;
	cd_itemstartx = gl_charwide * 4;
	cd_laststartx = 0;

	cd_sizestartx = cd_itemstartx + gl_charwide * 14;
	cd_stimstartx = cd_sizestartx + gl_charwide * 11;
	cd_etimstartx = cd_stimstartx + gl_charwide * 10;
	cd_commstartx = cd_etimstartx + gl_charwide * 10;

	return(cd_titlhite + cd_infohite + cd_headhite + cd_sesshite);
}

/* -----------------------------------------------------------------------------
 */

void	cd_set_parameters()
{
register CDROM	*docu = gl_actdocu;

	cd_docuwide = cd_commstartx + gl_charwide * MAX_COMLEN;
	cd_docuhite = cd_y_coordinate(docu->LastItem);
	cd_visx = 0L;
	cd_visy = 0L;

	cd_set_horzslider();
	cd_set_vertslider();
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_set_windareas()
{
register CDROM	*docu = gl_actdocu;
register int	rasx;
register int	rasy;
register int	rasw;
register int	rash;

	wind_get(gl_acthandle, WF_CURRXYWH, &gl_windrect.g_x, &gl_windrect.g_y, &gl_windrect.g_w, &gl_windrect.g_h);
	wind_get(gl_acthandle, WF_WORKXYWH, &cd_totlrect.g_x, &cd_totlrect.g_y, &cd_totlrect.g_w, &cd_totlrect.g_h);

	rasx = cd_totlrect.g_x;
	rasy = cd_totlrect.g_y;
	rasw = cd_totlrect.g_w;
	rash = cd_totlrect.g_h;

	cd_titlrect.g_x = rasx;
	cd_titlrect.g_y = rasy;
	cd_titlrect.g_w = rasw;
	cd_titlrect.g_h = cd_titlhite;

	rasy += cd_titlhite;
	rash -= cd_titlhite;
	if (rash < 0)	rash = 0;

	cd_inforect.g_x = rasx;
	cd_inforect.g_y = rasy;
	cd_inforect.g_w = rasw;
	cd_inforect.g_h = cd_infohite;

	rasy += cd_infohite;
	rash -= cd_infohite;
	if (rash < 0)	rash = 0;

	cd_workrect.g_x = rasx;
	cd_workrect.g_y = rasy;
	cd_workrect.g_w = rasw;
	cd_workrect.g_h = rash;

	cd_headrect.g_x = rasx;
	cd_headrect.g_y = rasy;
	cd_headrect.g_w = rasw;
	cd_headrect.g_h = cd_headhite;

	rasy += cd_headhite;
	rash -= cd_headhite;
	if (rash < 0)	rash = 0;

	cd_datarect.g_x = rasx;
	cd_datarect.g_y = rasy;
	cd_datarect.g_w = rasw;
	cd_datarect.g_h = rash;

/* set the document width and height */

	cd_docuwide = cd_commstartx + gl_charwide * MAX_COMLEN;
	cd_docuhite = cd_y_coordinate(docu->LastItem);

/* update the visual x/y parameters */

	cd_set_maxvisxy();

/* force a redraw message for the title area */

	st_force_redraw(gl_acthandle, &cd_titlrect);
	return;
}

/* -----------------------------------------------------------------------------
 * Redraw the given WP document window
 */

void	cd_redraw(drawrect)
register GRECT	*drawrect;			/* rectangle to be redrawn */
{
	cd_display_title(drawrect);
	cd_display_info(drawrect);
	cd_display_header(drawrect);
	cd_display(drawrect);
	cd_select(drawrect);
	cd_currsel(drawrect);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_wind_arrowed(type)
int	type;
{
	switch(type)
	{
	case 0 : cd_page_up();		break;
	case 1 : cd_page_down();	break;

	case 2 : cd_row_up();		break;
	case 3 : cd_row_down();		break;

	case 4 : cd_page_left();	break;
	case 5 : cd_page_right();	break;

	case 6 : cd_col_left();		break;
	case 7 : cd_col_right();	break;
	}
	return;
} 

/* -----------------------------------------------------------------------------
 */

void	cd_wind_hslide(pos)
register int	pos;
{
register long	newx;
int	old_pos;
int	junk;

	wind_get(gl_acthandle, WF_HSLIDE, &old_pos, &junk, &junk, &junk);
	if (old_pos == pos)
		return;

	newx = (long)(((double)pos * (double)(cd_docuwide - (long)cd_datarect.g_w)) / 1000.0 + 0.5);
	if (newx < 0L)	newx = 0L;

	cd_scroll_h(cd_visx - newx);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_wind_vslide(pos)
register int	pos;
{
register long	newy;
int	old_pos;
int	junk;

	wind_get(gl_acthandle, WF_VSLIDE, &old_pos, &junk, &junk, &junk);
	if (old_pos == pos)
		return;

	newy = (long)(((double)pos * (double)(cd_docuhite - (long)cd_datarect.g_h)) / 1000.0 + 0.5);
	if (newy < 0L)	newy = 0L;

	cd_scroll_v(cd_visy - newy);
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	void	cd_page_up()
{
	cd_scroll_v((long)( cd_datarect.g_h));
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_page_down()
{
	cd_scroll_v((long)(-cd_datarect.g_h));
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_row_up()
{
register ITEM	*item;
register long	liny;
register long	newy;
register long	shift;

	liny = cd_visy;
	newy = cd_inv_coordinate(liny);
	item = cd_inv_item;

	shift = liny - (newy - (long)item->Hite);
	if (shift == 0L)
	{
		item = item->prev;
		if (item == NULL)
			return;

		shift = item->Hite;
	}
	cd_scroll_v(shift);
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_row_down()
{
register ITEM	*item;
register long	liny;
register long	newy;
register long	shift;

	liny = cd_visy + (long)cd_datarect.g_h;
	newy = cd_inv_coordinate(liny);
	item = cd_inv_item;

	shift = newy - liny;
	if (shift == 0L)
	{
		item = item->next;
		if (item == NULL)
			return;

		shift = item->Hite;
	}
	cd_scroll_v(-shift);
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_page_left()
{
	cd_scroll_h((long)( cd_datarect.g_w));
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_page_right()
{
	cd_scroll_h((long)(-cd_datarect.g_w));
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_col_left()
{
	cd_scroll_h((long)( gl_charwide));
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	cd_col_right()
{
	cd_scroll_h((long)(-gl_charwide));
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_scroll_v(shift)
register long	shift;
{
register long	newy;
register long	vbound;
GRECT	clip;

	newy = cd_visy - shift;
	vbound = cd_docuhite - (long)cd_datarect.g_h;
	if (vbound <= 0L)
		return;

	if (newy > vbound)	newy = vbound;
	if (newy < 0L)		newy = 0L;

	shift = cd_visy - newy;
	if (!shift)
		return;

	cd_visy = newy;

	/* set the first visible session/track/item in window */

	clip.g_x = cd_workrect.g_x;
	clip.g_w = cd_workrect.g_w;

	clip.g_y = cd_datarect.g_y;
	clip.g_h = cd_datarect.g_h;

	cd_set_vertslider();
	st_scroll_v(gl_acthandle, &clip, shift);
	if (clip.g_h)
	{
		cd_redraw(&clip);
	}
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_scroll_h(shift)
register long	shift;
{
register long	newx;
register long	hbound;
GRECT	clip;

	newx = cd_visx - shift;
	hbound = cd_docuwide - (long)cd_datarect.g_w;
	if (hbound <= 0L)
		return;

	if (newx > hbound)	newx = hbound;
	if (newx < 0L)		newx = 0L;

	shift = cd_visx - newx;
	if (!shift)
		return;

	cd_visx = newx;

	clip.g_x = cd_workrect.g_x;
	clip.g_w = cd_workrect.g_w;

	clip.g_y = cd_workrect.g_y;
	clip.g_h = cd_workrect.g_h;

	cd_set_horzslider();
	st_scroll_h(gl_acthandle, &clip, shift);
	if (clip.g_w)
	{
		cd_redraw(&clip);
	}
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_set_vertslider()
{
register long	docuhite = cd_docuhite;
register long	windhite = (long)cd_datarect.g_h;
register long	visy = cd_visy;
register int	pos;
int	old_pos, junk;

	if (docuhite < windhite)	docuhite = windhite;
	pos = (int)(((double)windhite / (double)docuhite) * 1000.0 + 0.5);

	wind_get(gl_acthandle, WF_VSLSIZE, &old_pos, &junk, &junk, &junk);
	if (old_pos != pos) wind_set(gl_acthandle, WF_VSLSIZE, pos, 0, 0, 0);

	pos = (int)(((double)visy / (double)(docuhite - windhite)) * 1000.0 + 0.5);
	if (pos < 0)	pos = 0;

	wind_get(gl_acthandle, WF_VSLIDE, &old_pos, &junk, &junk, &junk);
	if (old_pos != pos) wind_set(gl_acthandle, WF_VSLIDE, pos, 0, 0, 0);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_set_horzslider()
{
register long	docuwide = cd_docuwide;
register long	windwide = (long)cd_datarect.g_w;
register long	visx = cd_visx;
register int	pos;
int	old_pos, junk;

	if (docuwide < windwide)	docuwide = windwide;
	pos = (int)(((double)windwide / (double)docuwide) * 1000.0 + 0.5);

	wind_get(gl_acthandle, WF_HSLSIZE, &old_pos, &junk, &junk, &junk);
	if (old_pos != pos) wind_set(gl_acthandle, WF_HSLSIZE, pos, 0, 0, 0);

	pos = (int)(((double)visx / (double)(docuwide - windwide)) * 1000.0 + 0.5);

	wind_get(gl_acthandle, WF_HSLIDE, &old_pos, &junk, &junk, &junk);
	if (old_pos != pos) wind_set(gl_acthandle, WF_HSLIDE, pos, 0, 0, 0);
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_scroll_disp()
{
register CDROM	*docu = gl_actdocu;
register ITEM	*item = docu->SelItem;
register long	liny1;
register long	liny2;
register long	y1;
register long	y2;
register long	shift;
 
	liny2 = cd_y_coordinate(item);
	liny1 = liny2 - item->Hite;

	y1 = cd_visy;
	y2 = cd_visy + cd_datarect.g_h;

	shift = 0L;
	if (liny1 < y1)
	{
		shift = y1 - liny1;
	}
	else
	if (liny2 > y2)
	{
		shift = y2 - liny2;
	}
	cd_scroll_v(shift);
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_set_maxvisxy()
{
register long	newx;
register long	newy;

	newx = cd_docuwide - (long)cd_datarect.g_w;
	if (newx < 0L)		newx = 0L;
	if (cd_visx > newx)	cd_visx = newx;
	cd_set_horzslider();

	newy = cd_docuhite - (long)cd_datarect.g_h;
	if (newy < 0L)		newy = 0L;
	if (cd_visy > newy)	cd_visy = newy;
	cd_set_vertslider();
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_visxy_update()
{
register long	oldx = cd_visx;
register long	oldy = cd_visy;

	cd_set_maxvisxy();
	if ((cd_visx == oldx) && (cd_visy == oldy))
		return;

	cd_redraw(&cd_totlrect);
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_setmenu_rest()
{
register OBJECT	*menu = gl_actmenu;
register int	flag = TRUE;

	menu_ienable(menu, CD_NEW, flag);
	menu_ienable(menu, CD_OPEN, flag);
	menu_ienable(menu, CD_SAVE, flag);
	menu_ienable(menu, CD_SAVAS, flag);
	menu_ienable(menu, CD_LOGFL, flag);
	menu_ienable(menu, CD_EMULT, flag);
	menu_ienable(menu, CD_ROMNM, flag);
	menu_ienable(menu, CD_PREF, flag);
	menu_ienable(menu, CD_PRELD, flag);
	menu_ienable(menu, CD_LTNCY, flag);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_setmenu_empty()
{
register OBJECT	*menu = gl_actmenu;
register CDROM	*docu = gl_actdocu;
register int	flag;

	flag = ((docu->NoItems > 0) ? TRUE : FALSE);
	menu_ienable(menu, CD_FILST, flag);
	menu_ienable(menu, CD_SELAL, flag);
	menu_ienable(menu, CD_FIND , flag);

	flag = ((flag && cd_srchflag) ? TRUE : FALSE);
	menu_ienable(menu, CD_FINDN, flag);

	flag = ((docu->NoSessions > 1) ? TRUE : FALSE);
	menu_ienable(menu, CD_GOSES, flag);

	flag = ((docu->NoTracks > 1) ? TRUE : FALSE);
	menu_ienable(menu, CD_GOTRK, flag);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_setmenu_insert()
{
register OBJECT	*menu = gl_actmenu;
register CDROM	*docu = gl_actdocu;
register ITEM	*item = docu->SelItem;
register int	NoSessions = docu->NoSessions;
register int	NoTracks = docu->NoTracks;
register int	NoItems = docu->NoItems;
register unsigned int	settings = item->Settings;
register int	flag;

	flag = ((settings & IT_LAST) ? FALSE : TRUE);
	menu_ienable(menu, CD_COMNT, flag);

	     if (NoSessions >= MAX_SESSIONS)	flag = FALSE;
	else if (settings & IT_ITEM)		flag = FALSE;
	else if (settings & IT_TRACK)		flag = ((item->prev->Settings & IT_SESSION) ? FALSE : TRUE);
	else if (settings & IT_SESSION)		flag = (((NoTracks >= MAX_TRACKS) || (NoItems >= MAX_ITEMS)) ? FALSE : TRUE);
	else if (settings & IT_LAST)		flag = (((NoTracks >= MAX_TRACKS) || (NoItems >= MAX_ITEMS)) ? FALSE : TRUE);
	else					flag = FALSE;
	menu_ienable(menu, CD_INSSS, flag);

	     if (NoTracks >= MAX_TRACKS)	flag = FALSE;
	else if (settings & IT_ITEM)		flag = ((item->prev->Settings & IT_TRACK) ? FALSE : TRUE);
	else if (settings & IT_TRACK)		flag = (((NoItems >= MAX_ITEMS)) ? FALSE : TRUE);
	else if (settings & IT_SESSION)		flag = (((NoItems >= MAX_ITEMS) || (item == docu->FirstItem)) ? FALSE : TRUE);
	else if (settings & IT_LAST)		flag = (((NoItems >= MAX_ITEMS) || (item == docu->FirstItem)) ? FALSE : TRUE);
	else					flag = FALSE;
	menu_ienable(menu, CD_INSTR, flag);

	     if (NoItems >= MAX_ITEMS)		flag = FALSE;
	else if (settings & IT_ITEM)		flag = TRUE;
	else if (settings & IT_TRACK)		flag = ((item->prev->Settings & IT_SESSION) ? FALSE : TRUE);
	else if (settings & IT_SESSION)		flag = ((item == docu->FirstItem) ? FALSE : TRUE);
	else if (settings & IT_LAST)		flag = ((item == docu->FirstItem) ? FALSE : TRUE);
	else					flag = FALSE;
	menu_ienable(menu, CD_INSFL, flag);

	flag = ((item->Settings & IT_ITEM) ? TRUE : FALSE);
	menu_ienable(menu, CD_EDFIL, flag);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_setmenu_select()
{
register OBJECT	*menu = gl_actmenu;
register int	flag;

	flag = ((gl_actdocu->NoSelItems > 0) ? TRUE : FALSE);
	menu_ienable(menu, CD_CUT, flag);
	menu_ienable(menu, CD_COPY, flag);
	menu_ienable(menu, CD_DELET, flag);

	menu_ienable(menu, CD_PASTE, st_clip_gettype());
	return;
}

/* -----------------------------------------------------------------------------
 */

void	cd_setmenu_undo()
{
register int	flag;

	flag = ((undo_gl_actdocu == NULL) ? FALSE : TRUE);
	menu_ienable(gl_actmenu, CD_UNDO, flag);
	return;
}

/******************************************************************************/
/* End of CD_INIT.C							      */
/******************************************************************************/
