/******************************************************************************/
/* CD_MENU.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"
#include "st_aes.h"
#include "st_lib.h"
#include "cd_bind.h"

#include "cdmonusa.h"

/* -------------------------------------------------------------------------- */

extern	unsigned char	gl_pathname[];

extern	CDROM	*gl_actdocu;

extern	int	cd_srchflag;
extern	unsigned char	cd_srchstrs[];

/* -------------------------------------------------------------------------- */

unsigned char	cd_logpath[ MAX_PATHNAME ];

/* -------------------------------------------------------------------------- */
/* default device id */

unsigned int	cd_defdevno;	/* default SCSI device number */

/* different CDROM latency */

unsigned int	cd_spinup;
unsigned int	cd_spinup_addsession;
unsigned int	cd_spinup_addtrack;

unsigned int	cd_stopdisc;
unsigned int	cd_stopdisc_middle;
unsigned int	cd_stopdisc_end;

unsigned int	cd_pause_toready;
unsigned int	cd_unpause_toready;

unsigned int	cd_shortseek_1sthalf;
unsigned int	cd_shortseek_2ndhalf;

unsigned int	cd_longseek_00to20;
unsigned int	cd_longseek_21to40;
unsigned int	cd_longseek_41to60;
unsigned int	cd_longseek_61to74;

unsigned int	cd_longseek_00to74;
unsigned int	cd_longseek_00to74_addsession;

unsigned int	cd_longseek_00to35;
unsigned int	cd_longseek_00to35_addsession;

/* different lead in/out and pause time periods for CDROMs */

unsigned long	cd_cdrmlead;	/* cd rom leadin in frames */
unsigned long	cd_cdrmtail;	/* cd rom leadout in frames */

unsigned long	cd_sesslead;	/* session leadin in frames */
unsigned long	cd_sesstail;	/* session leadout in frames */

unsigned long	cd_traklead;	/* track leadin in frames */
unsigned long	cd_traktail;	/* track leadout in frames */

/* preload buffer size during seek and data rate from SCSI drive */

unsigned long	cd_preload_buffsize;
unsigned long	cd_preload_readrate;

/* -------------------------------------------------------------------------- */

static	unsigned char	*cd_logfile = "CDROM.LOG";

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_init_preferences()
{
register unsigned char	*cp;

	strcpy(cd_logpath, gl_pathname);
	if (cp = strrchr(cd_logpath, '\\'))
		cp[1] = '\0';
	strcat(cd_logpath, cd_logfile);

/* initialise default SCSI device used */

	cd_defdevno = 0;

/* initialise latency parameters - in msecs */

	cd_spinup			= 4000;
	cd_spinup_addsession		= 500;
	cd_spinup_addtrack		= 1000;

	cd_stopdisc			= 1000;
	cd_stopdisc_middle		= 1000;
	cd_stopdisc_end			= 2000;

	cd_pause_toready		= 100;
	cd_unpause_toready		= 167;

	cd_shortseek_1sthalf		= 250;
	cd_shortseek_2ndhalf		= 375;

	cd_longseek_00to20		= 500;
	cd_longseek_21to40		= 625;
	cd_longseek_41to60		= 750;
	cd_longseek_61to74		= 1000;

	cd_longseek_00to74		= 1500;
	cd_longseek_00to74_addsession	= 250;

	cd_longseek_00to35		= 750;
	cd_longseek_00to35_addsession	= 250;

/* initialise cdrom/session/track leadin and leadout parameters */

	cd_cdrmlead = 2l * MAX_SECSIZE;
	cd_cdrmtail = 2l * MAX_SECSIZE;

	cd_sesslead = 2l * MAX_SECSIZE;
	cd_sesstail = 2l * MAX_SECSIZE;

	cd_traklead = 2l * MAX_SECSIZE;
	cd_traktail = 2l * MAX_SECSIZE;

/* initialise pre-load buffer sizes while pretending to seek on CD */

	cd_preload_buffsize = 0;
	cd_preload_readrate = 0;

/* initialise search buffer and flag */

	cd_srchflag = FALSE;
	strcpy(cd_srchstrs, "");
	return;
}

/* -----------------------------------------------------------------------------
 */

int	cd_save_preferences(fp)
register FILE	*fp;
{
unsigned char	header[80];

	header[0] = (unsigned char)strlen(cd_logpath);
	header[1] = (unsigned char)strlen(cd_srchstrs);
	header[2] = (unsigned char)cd_srchflag;
	header[3] = (unsigned char)cd_defdevno;

	(*((unsigned int *)(&header[ 4]))) = cd_spinup;
	(*((unsigned int *)(&header[ 6]))) = cd_spinup_addsession;
	(*((unsigned int *)(&header[ 8]))) = cd_spinup_addtrack;

	(*((unsigned int *)(&header[10]))) = cd_stopdisc;
	(*((unsigned int *)(&header[12]))) = cd_stopdisc_middle;
	(*((unsigned int *)(&header[14]))) = cd_stopdisc_end;

	(*((unsigned int *)(&header[16]))) = cd_pause_toready;
	(*((unsigned int *)(&header[18]))) = cd_unpause_toready;

	(*((unsigned int *)(&header[20]))) = cd_shortseek_1sthalf;
	(*((unsigned int *)(&header[22]))) = cd_shortseek_2ndhalf;

	(*((unsigned int *)(&header[24]))) = cd_longseek_00to20;
	(*((unsigned int *)(&header[26]))) = cd_longseek_21to40;
	(*((unsigned int *)(&header[28]))) = cd_longseek_41to60;
	(*((unsigned int *)(&header[30]))) = cd_longseek_61to74;

	(*((unsigned int *)(&header[32]))) = cd_longseek_00to74;
	(*((unsigned int *)(&header[34]))) = cd_longseek_00to74_addsession;

	(*((unsigned int *)(&header[36]))) = cd_longseek_00to35;
	(*((unsigned int *)(&header[38]))) = cd_longseek_00to35_addsession;

	(*((unsigned long *)(&header[40]))) = cd_cdrmlead;
	(*((unsigned long *)(&header[44]))) = cd_cdrmtail;

	(*((unsigned long *)(&header[48]))) = cd_sesslead;
	(*((unsigned long *)(&header[52]))) = cd_sesstail;

	(*((unsigned long *)(&header[56]))) = cd_traklead;
	(*((unsigned long *)(&header[60]))) = cd_traktail;

	(*((unsigned long *)(&header[64]))) = cd_preload_buffsize;
	(*((unsigned long *)(&header[68]))) = cd_preload_readrate;

	if (!fwrite(header, (size_t)1, (size_t)72, fp))
		return(FALSE);

	if (header[0] && !fwrite(cd_logpath, (size_t)1, (size_t)header[0], fp))
		return(FALSE);

	if (header[1] && !fwrite(cd_srchstrs, (size_t)1, (size_t)header[1], fp))
		return(FALSE);

	return(TRUE);
}

/* -----------------------------------------------------------------------------
 */

int	cd_load_preferences(fp)
register FILE	*fp;
{
unsigned char	header[80];

	if (!fread(header, (size_t)1, (size_t)72, fp))
		return(FALSE);

	if ((header[0] >= MAX_PATHNAME) || (header[1] >= MAX_SRCHSTRS))
		return(FALSE);

	if (header[2] > 1)
		return(FALSE);
	if (header[3] >= 8)
		return(FALSE);

	cd_srchflag			= (unsigned int)header[2];
	cd_defdevno			= (unsigned int)header[3];

	cd_spinup			= (*((unsigned int *)(&header[ 4])));
	cd_spinup_addsession		= (*((unsigned int *)(&header[ 6])));
	cd_spinup_addtrack		= (*((unsigned int *)(&header[ 8])));

	cd_stopdisc			= (*((unsigned int *)(&header[10])));
	cd_stopdisc_middle		= (*((unsigned int *)(&header[12])));
	cd_stopdisc_end			= (*((unsigned int *)(&header[14])));

	cd_pause_toready		= (*((unsigned int *)(&header[16])));
	cd_unpause_toready		= (*((unsigned int *)(&header[18])));

	cd_shortseek_1sthalf		= (*((unsigned int *)(&header[20])));
	cd_shortseek_2ndhalf		= (*((unsigned int *)(&header[22])));

	cd_longseek_00to20		= (*((unsigned int *)(&header[24])));
	cd_longseek_21to40		= (*((unsigned int *)(&header[26])));
	cd_longseek_41to60		= (*((unsigned int *)(&header[28])));
	cd_longseek_61to74		= (*((unsigned int *)(&header[30])));

	cd_longseek_00to74		= (*((unsigned int *)(&header[32])));
	cd_longseek_00to74_addsession	= (*((unsigned int *)(&header[34])));

	cd_longseek_00to35		= (*((unsigned int *)(&header[36])));
	cd_longseek_00to35_addsession	= (*((unsigned int *)(&header[38])));

	cd_cdrmlead			= (*((unsigned long *)(&header[40])));
	cd_cdrmtail			= (*((unsigned long *)(&header[44])));

	cd_sesslead			= (*((unsigned long *)(&header[48])));
	cd_sesstail			= (*((unsigned long *)(&header[52])));

	cd_traklead			= (*((unsigned long *)(&header[56])));
	cd_traktail			= (*((unsigned long *)(&header[60])));

	cd_preload_buffsize		= (*((unsigned long *)(&header[64])));
	cd_preload_readrate		= (*((unsigned long *)(&header[68])));

	if (header[0] && !fread(cd_logpath, (size_t)1, (size_t)header[0], fp))
		return(FALSE);

	if (header[1] && !fread(cd_srchstrs, (size_t)1, (size_t)header[1], fp))
		return(FALSE);

	cd_logpath[ header[0] ] = '\0';
	cd_srchstrs[ header[1] ] = '\0';
	return(TRUE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_set_logfilename()
{
unsigned char	filename[ MAX_PATHNAME ];

	strcpy(filename, cd_logpath);
	if (st_get_filename(1, AWRITE, filename, ".LOG", FSEL_LOGFILENAME))
	{
		strcpy(cd_logpath, filename);
	}
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_dial_latency()
{
register OBJECT	*Tree;
register int	Respons;

	Tree = st_objc_get_tree(LATENCY);

	sprintf(TePtext(LISPINUP), "%4d", cd_spinup);
	sprintf(TePtext(LSPNSESS), "%4d", cd_spinup_addsession);
	sprintf(TePtext(LSPNTRAK), "%4d", cd_spinup_addtrack);

	sprintf(TePtext(LSTOPDSC), "%4d", cd_stopdisc);
	sprintf(TePtext(LSTOPMID), "%4d", cd_stopdisc_middle);
	sprintf(TePtext(LSTOPEND), "%4d", cd_stopdisc_end);

	sprintf(TePtext(LPAUSERD), "%4d", cd_pause_toready);
	sprintf(TePtext(LUNPAUSE), "%4d", cd_unpause_toready);

	sprintf(TePtext(LSEEK1HF), "%4d", cd_shortseek_1sthalf);
	sprintf(TePtext(LSEEK2HF), "%4d", cd_shortseek_2ndhalf);

	sprintf(TePtext(LS00TO20), "%4d", cd_longseek_00to20);
	sprintf(TePtext(LS21TO40), "%4d", cd_longseek_21to40);
	sprintf(TePtext(LS41TO60), "%4d", cd_longseek_41to60);
	sprintf(TePtext(LS61TO74), "%4d", cd_longseek_61to74);

	sprintf(TePtext(LS00TO74), "%4d", cd_longseek_00to74);
	sprintf(TePtext(LS074SES), "%4d", cd_longseek_00to74_addsession);

	sprintf(TePtext(LS00TO35), "%4d", cd_longseek_00to35);
	sprintf(TePtext(LS035SES), "%4d", cd_longseek_00to35_addsession);

	st_objc_draw_tree(Tree);
	Respons = form_do(Tree, LISPINUP);
	Tree[ Respons ].ob_state &= ~SELECTED;
	st_objc_erase_tree(Tree);

	if (Respons == LATNCYCN)
		return;

	cd_spinup			= atoi(TePtext(LISPINUP));
	cd_spinup_addsession		= atoi(TePtext(LSPNSESS));
	cd_spinup_addtrack		= atoi(TePtext(LSPNTRAK));

	cd_stopdisc			= atoi(TePtext(LSTOPDSC));
	cd_stopdisc_middle		= atoi(TePtext(LSTOPMID));
	cd_stopdisc_end			= atoi(TePtext(LSTOPEND));

	cd_pause_toready		= atoi(TePtext(LPAUSERD));
	cd_unpause_toready		= atoi(TePtext(LUNPAUSE));

	cd_shortseek_1sthalf		= atoi(TePtext(LSEEK1HF));
	cd_shortseek_2ndhalf		= atoi(TePtext(LSEEK2HF));

	cd_longseek_00to20		= atoi(TePtext(LS00TO20));
	cd_longseek_21to40		= atoi(TePtext(LS21TO40));
	cd_longseek_41to60		= atoi(TePtext(LS41TO60));
	cd_longseek_61to74		= atoi(TePtext(LS61TO74));

	cd_longseek_00to74		= atoi(TePtext(LS00TO74));
	cd_longseek_00to74_addsession	= atoi(TePtext(LS074SES));

	cd_longseek_00to35		= atoi(TePtext(LS00TO35));
	cd_longseek_00to35_addsession	= atoi(TePtext(LS035SES));

	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_dial_preferences()
{
register OBJECT	*Tree;
register int	Respons;

	Tree = st_objc_get_tree(PREFDIAL);

	sprintf(TePtext(CDLEADIN), "%6ld", cd_cdrmlead);
	sprintf(TePtext(CDLEADOT), "%6ld", cd_cdrmtail);

	sprintf(TePtext(SSLEADIN), "%6ld", cd_sesslead);
	sprintf(TePtext(SSLEADOT), "%6ld", cd_sesstail);

	sprintf(TePtext(TRLEADIN), "%6ld", cd_traklead);
	sprintf(TePtext(TRLEADOT), "%6ld", cd_traktail);

	sprintf(TePtext(PREFSCSI), "%1d", cd_defdevno);

	st_objc_draw_tree(Tree);
	Respons = form_do(Tree, SSLEADIN);
	Tree[ Respons ].ob_state &= ~SELECTED;
	st_objc_erase_tree(Tree);

	if (Respons == PREFCANC)
		return;

	cd_cdrmlead	= atol(TePtext(CDLEADIN));
	cd_cdrmtail	= atol(TePtext(CDLEADOT));

	cd_sesslead	= atol(TePtext(SSLEADIN));
	cd_sesstail	= atol(TePtext(SSLEADOT));

	cd_traklead	= atol(TePtext(TRLEADIN));
	cd_traktail	= atol(TePtext(TRLEADOT));

	cd_defdevno	= atoi(TePtext(PREFSCSI));

	cd_update_time(gl_actdocu);
	cd_update_disp();
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_dial_preloadpref()
{
register OBJECT	*Tree;
register int	Respons;

	Tree = st_objc_get_tree(PREFLOAD);

	sprintf(TePtext(PFLDRATE), "%6ld", (cd_preload_readrate / 1024));
	sprintf(TePtext(PFLDBUFF), "%6ld", (cd_preload_buffsize / 512));

	st_objc_draw_tree(Tree);
	Respons = form_do(Tree, PFLDBUFF);
	Tree[ Respons ].ob_state &= ~SELECTED;
	st_objc_erase_tree(Tree);

	if (Respons == PFLDCANC)
		return;

	cd_preload_readrate	= atol(TePtext(PFLDRATE)) * 1024;

	cd_preload_buffsize	= atol(TePtext(PFLDBUFF)) * 512;
	if (cd_preload_buffsize > PRELOAD_BUFFSIZE) cd_preload_buffsize = PRELOAD_BUFFSIZE;
	return;
}

/******************************************************************************/
/* End of CD_MENU.C							      */
/******************************************************************************/

